@@ -1,8 +1,5 @@
 package Devel::PatchPerl;
-{
-  $Devel::PatchPerl::VERSION = '0.76';
-}
-
+$Devel::PatchPerl::VERSION = '1.28';
 # ABSTRACT: Patch perl source a la Devel::PPPort's buildperl.pl
 
 use strict;
@@ -10,7 +7,6 @@ use warnings;
 use File::pushd qw[pushd];
 use File::Spec;
 use IO::File;
-use IPC::Cmd qw[can_run run];
 use Devel::PatchPerl::Hints qw[hint_file];
 use Module::Pluggable search_path => ['Devel::PatchPerl::Plugin'];
 use vars qw[@ISA @EXPORT_OK];
@@ -18,7 +14,7 @@ use vars qw[@ISA @EXPORT_OK];
 @ISA       = qw(Exporter);
 @EXPORT_OK = qw(patch_source);
 
-my $patch_exe = can_run('patch');
+my $patch_exe = _can_run('patch') || _can_run('gpatch');
 
 my @patch = (
   {
@@ -35,6 +31,15 @@ my @patch = (
               [ \&_patch_db, 1 ],
             ],
   },
+  {
+    perl => [
+              qr/^5\.6\.[1-2]$/,
+              qr/^5\.7\.[0-1]$/,
+            ],
+    subs => [
+              [ \&_patch_makefile_sh_phony ],
+            ],
+  },
   {
     perl => [
             qw/
@@ -97,6 +102,15 @@ my @patch = (
               [ \&_patch_makedepend_lc ],
             ],
   },
+  {
+    perl => [
+              qr/^5\.6\.[0-2]$/,
+            ],
+    subs => [
+              [ \&_patch_conf_gconvert ],
+              [ \&_patch_sort_N ],
+            ],
+  },
   {
     perl => [
               '5.8.0',
@@ -110,6 +124,8 @@ my @patch = (
               qr/.*/,
             ],
     subs => [
+              [ \&_patch_conf_solaris ],
+              [ \&_patch_bitrig ],
               [ \&_patch_hints ],
             ],
   },
@@ -144,6 +160,43 @@ my @patch = (
     perl => [ qr/^5\.8\.9$/, ],
     subs => [ [ \&_patch_589_perlio_c ], ],
   },
+  {
+    perl => [ qr/^5\.8\.[89]$/ ],
+    subs => [ [ \&_patch_hsplit_rehash_58 ] ],
+  },
+  {
+    perl => [
+              qr/^5\.10\.1$/,
+              qr/^5\.12\.5$/,
+            ],
+    subs => [ [ \&_patch_hsplit_rehash_510 ] ],
+  },
+  {
+    perl => [
+              qr/^5\.18\.0$/,
+            ],
+    subs => [ [ \&_patch_regmatch_pointer_5180 ] ],
+  },
+  {
+    perl => [
+              qr/^5\.20\.0$/,
+            ],
+    subs => [ [ \&_patch_cow_speed ] ],
+  },
+  {
+    perl => [
+              qr/^5\.6\.[012]$/,
+              qr/^5\.8\.[89]$/,
+              qr/^5\.10\.[01]$/,
+            ],
+    subs => [ [ \&_patch_preprocess_options ] ],
+  },
+  {
+    perl => [
+              qr/^5\.18\.3$/,
+            ],
+    subs => [ [ \&_patch_5183_metajson ] ],
+  },
 );
 
 sub patch_source {
@@ -204,6 +257,39 @@ sub _process_plugin {
   return 1;
 }
 
+sub _can_run {
+    my $command = shift;
+
+    # a lot of VMS executables have a symbol defined
+    # check those first
+    if ( $^O eq 'VMS' ) {
+        require VMS::DCLsym;
+        my $syms = VMS::DCLsym->new;
+        return $command if scalar $syms->getsym( uc $command );
+    }
+
+    require File::Spec;
+    require ExtUtils::MakeMaker;
+
+    my @possibles;
+
+    if( File::Spec->file_name_is_absolute($command) ) {
+        return MM->maybe_command($command);
+
+    } else {
+        for my $dir (
+            File::Spec->path,
+            File::Spec->curdir
+        ) {
+            next if ! $dir || ! -d $dir;
+            my $abs = File::Spec->catfile( $^O eq 'MSWin32' ? Win32::GetShortPathName( $dir ) : $dir, $command);
+            push @possibles, $abs if $abs = MM->maybe_command($abs);
+        }
+    }
+    return @possibles if wantarray;
+    return shift @possibles;
+}
+
 sub _is
 {
   my($s1, $s2) = @_;
@@ -230,6 +316,7 @@ sub _patch
   my $diff = 'tmp.diff';
   _write_or_die($diff, $patch);
   die "No patch utility found\n" unless $patch_exe;
+  local $ENV{PATCH_GET} = 0; # I can't reproduce this at all, but meh.
   _run_or_die("$patch_exe -f -s -p0 <$diff");
   unlink $diff or die "unlink $diff: $!\n";
 }
@@ -244,7 +331,14 @@ sub _write_or_die
 sub _run_or_die
 {
   # print "[running @_]\n";
-  die unless scalar run( command => [ @_ ], verbose => 1 );
+  die unless system( @_ ) == 0;
+}
+
+sub determine_version {
+  my $src = shift;
+  $src = shift if eval { $src->isa(__PACKAGE__) };
+  $src = '.' unless $src;
+  _determine_version($src);
 }
 
 sub _determine_version {
@@ -265,29 +359,41 @@ sub _determine_version {
     if ( my @wotsits = grep { defined $defines{$_} } qw(PERL_REVISION PERL_VERSION PERL_SUBVERSION) ) {
       $version = join '.', map { $defines{$_} } @wotsits;
     }
+    elsif ( my @watsits = grep { defined $defines{$_} } qw(PATCHLEVEL SUBVERSION) ) {
+      $version = sprintf '5.%03d_%02d', map { $defines{$_} } @watsits;
+    }
     else {
-      $version = sprintf '5.%03d_%02d', map { $defines{$_} } qw(PATCHLEVEL SUBVERSION);
+      return;
     }
   }
   return $version;
 }
 
 sub _patch_hints {
-  return unless my ($file,$data) = hint_file();
-  my $path = File::Spec->catfile( 'hints', $file );
-  chmod 0644, $path or die "$!\n";
-  open my $fh, '>', $path or die "$!\n";
-  print $fh $data;
-  close $fh;
+  my @os;
+  push @os, $^O;
+  push @os, 'linux' if $^O eq 'gnukfreebsd'; # kfreebsd uses linux hints
+  foreach my $os ( @os ) {
+    return unless my ($file,$data) = hint_file( $os );
+    my $path = File::Spec->catfile( 'hints', $file );
+    if ( -e $path ) {
+      chmod 0644, $path or die "$!\n";
+    }
+    open my $fh, '>', $path or die "$!\n";
+    print $fh $data;
+    close $fh;
+  }
   return 1;
 }
 
 sub _patch_db
 {
   my $ver = shift;
-  print "patching ext/DB_File/DB_File.xs\n";
-  _run_or_die($^X, '-pi.bak', '-e', "s/<db.h>/<db$ver\\/db.h>/", 'ext/DB_File/DB_File.xs');
-  unlink 'ext/DB_File/DB_File.xs.bak' if -e 'ext/DB_File/DB_File.xs.bak';
+  for my $file ('ext/DB_File/DB_File.xs', 'Configure') {
+    print "patching $file\n";
+    _run_or_die($^X, '-pi.bak', '-e', "s/<db.h>/<db$ver\\/db.h>/", $file);
+    unlink "$file.bak" if -e "$file.bak";
+  }
 }
 
 sub _patch_doio
@@ -1682,6 +1788,61 @@ BADGER
   }
 }
 
+sub _patch_conf_gconvert
+{
+  my $perl = shift;
+  _patch(<<'END');
+--- Configure
++++ Configure
+@@ -7851,6 +7851,21 @@ int main()
+ 	Gconvert((DOUBLETYPE)0.1, 8, 0, buf);
+ 	checkit("0.1", buf);
+ 
++	Gconvert((DOUBLETYPE)0.01, 8, 0, buf);
++	checkit("0.01", buf);
++
++	Gconvert((DOUBLETYPE)0.001, 8, 0, buf);
++	checkit("0.001", buf);
++
++	Gconvert((DOUBLETYPE)0.0001, 8, 0, buf);
++	checkit("0.0001", buf);
++
++	Gconvert((DOUBLETYPE)0.00009, 8, 0, buf);
++	if (strlen(buf) > 5)
++	    checkit("9e-005", buf); /* for Microsoft ?? */
++	else
++	    checkit("9e-05", buf);
++
+ 	Gconvert((DOUBLETYPE)1.0, 8, 0, buf); 
+ 	checkit("1", buf);
+ 
+@@ -7889,6 +7904,19 @@ int main()
+ 	Gconvert((DOUBLETYPE)123.456, 8, 0, buf); 
+ 	checkit("123.456", buf);
+ 
++	/* Testing of 1e+129 in bigintpm.t must not get extra '.' here. */
++	Gconvert((DOUBLETYPE)1e34, 8, 0, buf);
++	/* 34 should be enough to scare even long double
++	 * places into using the e notation. */
++	if (strlen(buf) > 5)
++	    checkit("1e+034", buf); /* for Microsoft */
++	else
++	    checkit("1e+34", buf);
++
++	/* For Perl, if you add additional tests here, also add them to
++	 * t/base/num.t for benefit of platforms not using Configure or
++	 * overriding d_Gconvert */
++
+ 	exit(0);
+ }
+ EOP
+END
+}
+
+sub _patch_sort_N {
+  system($^X, '-pi.bak', '-e', 's!\$sort \-n \+1!(\$sort -n -k 2 2>/dev/null || \$sort -n +1)!', 'Configure');
+}
+
 sub _patch_archive_tar_tests
 {
   my $perl = shift;
@@ -1801,20 +1962,673 @@ sub _patch_589_perlio_c
 END
 }
 
+# http://perl5.git.perl.org/perl.git/commit/2674b61957c26a4924831d5110afa454ae7ae5a6
+sub _patch_hsplit_rehash_58
+{
+  my $perl = shift;
+
+  my $patch = <<'END';
+--- hv.c
++++ hv.c
+@@ -31,7 +31,8 @@ holds the key and hash value.
+ #define PERL_HASH_INTERNAL_ACCESS
+ #include "perl.h"
+ 
+-#define HV_MAX_LENGTH_BEFORE_SPLIT 14
++#define HV_MAX_LENGTH_BEFORE_REHASH 14
++#define SHOULD_DO_HSPLIT(xhv) ((xhv)->xhv_keys > (xhv)->xhv_max) /* HvTOTALKEYS(hv) > HvMAX(hv) */
+ 
+ STATIC void
+ S_more_he(pTHX)
+@@ -705,23 +706,8 @@ Perl_hv_common(pTHX_ HV *hv, SV *keysv, const char *key, STRLEN klen,
+ 	xhv->xhv_keys++; /* HvTOTALKEYS(hv)++ */
+ 	if (!counter) {				/* initial entry? */
+ 	    xhv->xhv_fill++; /* HvFILL(hv)++ */
+-	} else if (xhv->xhv_keys > (IV)xhv->xhv_max) {
++	} else if ( SHOULD_DO_HSPLIT(xhv) ) {
+ 	    hsplit(hv);
+-	} else if(!HvREHASH(hv)) {
+-	    U32 n_links = 1;
+-
+-	    while ((counter = HeNEXT(counter)))
+-		n_links++;
+-
+-	    if (n_links > HV_MAX_LENGTH_BEFORE_SPLIT) {
+-		/* Use only the old HvKEYS(hv) > HvMAX(hv) condition to limit
+-		   bucket splits on a rehashed hash, as we're not going to
+-		   split it again, and if someone is lucky (evil) enough to
+-		   get all the keys in one list they could exhaust our memory
+-		   as we repeatedly double the number of buckets on every
+-		   entry. Linear search feels a less worse thing to do.  */
+-		hsplit(hv);
+-	    }
+ 	}
+     }
+ 
+@@ -1048,7 +1034,7 @@ S_hsplit(pTHX_ HV *hv)
+ 
+ 
+     /* Pick your policy for "hashing isn't working" here:  */
+-    if (longest_chain <= HV_MAX_LENGTH_BEFORE_SPLIT /* split worked?  */
++    if (longest_chain <= HV_MAX_LENGTH_BEFORE_REHASH /* split worked?  */
+ 	|| HvREHASH(hv)) {
+ 	return;
+     }
+@@ -1966,8 +1952,8 @@ S_share_hek_flags(pTHX_ const char *str, I32 len, register U32 hash, int flags)
+ 	xhv->xhv_keys++; /* HvTOTALKEYS(hv)++ */
+ 	if (!next) {			/* initial entry? */
+ 	    xhv->xhv_fill++; /* HvFILL(hv)++ */
+-	} else if (xhv->xhv_keys > (IV)xhv->xhv_max /* HvKEYS(hv) > HvMAX(hv) */) {
+-		hsplit(PL_strtab);
++	} else if ( SHOULD_DO_HSPLIT(xhv) ) {
++            hsplit(PL_strtab);
+ 	}
+     }
+ 
+--- t/op/hash.t
++++ t/op/hash.t
+@@ -39,22 +39,36 @@ use constant THRESHOLD => 14;
+ use constant START     => "a";
+ 
+ # some initial hash data
+-my %h2 = map {$_ => 1} 'a'..'cc';
++my %h2;
++my $counter= "a";
++$h2{$counter++}++ while $counter ne 'cd';
+ 
+ ok (!Internals::HvREHASH(%h2), 
+     "starting with pre-populated non-pathological hash (rehash flag if off)");
+ 
+ my @keys = get_keys(\%h2);
++my $buckets= buckets(\%h2);
+ $h2{$_}++ for @keys;
++$h2{$counter++}++ while buckets(\%h2) == $buckets; # force a split
+ ok (Internals::HvREHASH(%h2), 
+-    scalar(@keys) . " colliding into the same bucket keys are triggering rehash");
++    scalar(@keys) . " colliding into the same bucket keys are triggering rehash after split");
++
++# returns the number of buckets in a hash
++sub buckets {
++    my $hr = shift;
++    my $keys_buckets= scalar(%$hr);
++    if ($keys_buckets=~m!/([0-9]+)\z!) {
++        return 0+$1;
++    } else {
++        return 8;
++    }
++}
+ 
+ sub get_keys {
+     my $hr = shift;
+ 
+     # the minimum of bits required to mount the attack on a hash
+     my $min_bits = log(THRESHOLD)/log(2);
+-
+     # if the hash has already been populated with a significant amount
+     # of entries the number of mask bits can be higher
+     my $keys = scalar keys %$hr;
+-- 
+1.7.4.1
+
+END
+
+  if ($perl =~ qr/^5\.8\.8$/) {
+    $patch =~ s/non-pathological/non-pathalogical/;
+    $patch =~ s/triggering/triggerring/;
+  }
+  _patch($patch);
+}
+
+# http://perl5.git.perl.org/perl.git/commit/f14269908e5f8b4cab4b55643d7dd9de577e7918
+# http://perl5.git.perl.org/perl.git/commit/9d83adcdf9ab3c1ac7d54d76f3944e57278f0e70
+sub _patch_hsplit_rehash_510 {
+  _patch(<<'END');
+--- ext/Hash-Util-FieldHash/t/10_hash.t
++++ ext/Hash-Util-FieldHash/t/10_hash.t
+@@ -46,15 +46,29 @@ use constant START     => "a";
+ 
+ # some initial hash data
+ fieldhash my %h2;
+-%h2 = map {$_ => 1} 'a'..'cc';
++my $counter= "a";
++$h2{$counter++}++ while $counter ne 'cd';
+ 
+ ok (!Internals::HvREHASH(%h2), 
+     "starting with pre-populated non-pathological hash (rehash flag if off)");
+ 
+ my @keys = get_keys(\%h2);
++my $buckets= buckets(\%h2);
+ $h2{$_}++ for @keys;
++$h2{$counter++}++ while buckets(\%h2) == $buckets; # force a split
+ ok (Internals::HvREHASH(%h2), 
+-    scalar(@keys) . " colliding into the same bucket keys are triggering rehash");
++    scalar(@keys) . " colliding into the same bucket keys are triggering rehash after split");
++
++# returns the number of buckets in a hash
++sub buckets {
++    my $hr = shift;
++    my $keys_buckets= scalar(%$hr);
++    if ($keys_buckets=~m!/([0-9]+)\z!) {
++        return 0+$1;
++    } else {
++        return 8;
++    }
++}
+ 
+ sub get_keys {
+     my $hr = shift;
+--- hv.c
++++ hv.c
+@@ -35,7 +35,8 @@ holds the key and hash value.
+ #define PERL_HASH_INTERNAL_ACCESS
+ #include "perl.h"
+ 
+-#define HV_MAX_LENGTH_BEFORE_SPLIT 14
++#define HV_MAX_LENGTH_BEFORE_REHASH 14
++#define SHOULD_DO_HSPLIT(xhv) ((xhv)->xhv_keys > (xhv)->xhv_max) /* HvTOTALKEYS(hv) > HvMAX(hv) */
+ 
+ static const char S_strtab_error[]
+     = "Cannot modify shared string table in hv_%s";
+@@ -818,23 +819,8 @@ Perl_hv_common(pTHX_ HV *hv, SV *keysv, const char *key, STRLEN klen,
+ 	xhv->xhv_keys++; /* HvTOTALKEYS(hv)++ */
+ 	if (!counter) {				/* initial entry? */
+ 	    xhv->xhv_fill++; /* HvFILL(hv)++ */
+-	} else if (xhv->xhv_keys > (IV)xhv->xhv_max) {
++	} else if ( SHOULD_DO_HSPLIT(xhv) ) {
+ 	    hsplit(hv);
+-	} else if(!HvREHASH(hv)) {
+-	    U32 n_links = 1;
+-
+-	    while ((counter = HeNEXT(counter)))
+-		n_links++;
+-
+-	    if (n_links > HV_MAX_LENGTH_BEFORE_SPLIT) {
+-		/* Use only the old HvKEYS(hv) > HvMAX(hv) condition to limit
+-		   bucket splits on a rehashed hash, as we're not going to
+-		   split it again, and if someone is lucky (evil) enough to
+-		   get all the keys in one list they could exhaust our memory
+-		   as we repeatedly double the number of buckets on every
+-		   entry. Linear search feels a less worse thing to do.  */
+-		hsplit(hv);
+-	    }
+ 	}
+     }
+ 
+@@ -1180,7 +1166,7 @@ S_hsplit(pTHX_ HV *hv)
+ 
+ 
+     /* Pick your policy for "hashing isn't working" here:  */
+-    if (longest_chain <= HV_MAX_LENGTH_BEFORE_SPLIT /* split worked?  */
++    if (longest_chain <= HV_MAX_LENGTH_BEFORE_REHASH /* split worked?  */
+ 	|| HvREHASH(hv)) {
+ 	return;
+     }
+@@ -2506,8 +2492,8 @@ S_share_hek_flags(pTHX_ const char *str, I32 len, register U32 hash, int flags)
+ 	xhv->xhv_keys++; /* HvTOTALKEYS(hv)++ */
+ 	if (!next) {			/* initial entry? */
+ 	    xhv->xhv_fill++; /* HvFILL(hv)++ */
+-	} else if (xhv->xhv_keys > (IV)xhv->xhv_max /* HvKEYS(hv) > HvMAX(hv) */) {
+-		hsplit(PL_strtab);
++	} else if ( SHOULD_DO_HSPLIT(xhv) ) {
++            hsplit(PL_strtab);
+ 	}
+     }
+ 
+diff --git a/t/op/hash.t b/t/op/hash.t
+index 9bde518..45eb782 100644
+--- t/op/hash.t
++++ t/op/hash.t
+@@ -39,22 +39,36 @@ use constant THRESHOLD => 14;
+ use constant START     => "a";
+ 
+ # some initial hash data
+-my %h2 = map {$_ => 1} 'a'..'cc';
++my %h2;
++my $counter= "a";
++$h2{$counter++}++ while $counter ne 'cd';
+ 
+ ok (!Internals::HvREHASH(%h2), 
+     "starting with pre-populated non-pathological hash (rehash flag if off)");
+ 
+ my @keys = get_keys(\%h2);
++my $buckets= buckets(\%h2);
+ $h2{$_}++ for @keys;
++$h2{$counter++}++ while buckets(\%h2) == $buckets; # force a split
+ ok (Internals::HvREHASH(%h2), 
+-    scalar(@keys) . " colliding into the same bucket keys are triggering rehash");
++    scalar(@keys) . " colliding into the same bucket keys are triggering rehash after split");
++
++# returns the number of buckets in a hash
++sub buckets {
++    my $hr = shift;
++    my $keys_buckets= scalar(%$hr);
++    if ($keys_buckets=~m!/([0-9]+)\z!) {
++        return 0+$1;
++    } else {
++        return 8;
++    }
++}
+ 
+ sub get_keys {
+     my $hr = shift;
+ 
+     # the minimum of bits required to mount the attack on a hash
+     my $min_bits = log(THRESHOLD)/log(2);
+-
+     # if the hash has already been populated with a significant amount
+     # of entries the number of mask bits can be higher
+     my $keys = scalar keys %$hr;
+-- 
+1.7.4.1
+
+
+END
+}
+
+sub _patch_bitrig {
+  return unless $^O eq 'bitrig';
+  my $perlver = shift;
+  my $num = _norm_ver( $perlver );
+  return unless $num < 5.019004;
+  unless ( $num < 5.00800 ) {
+  _patch(<<'BOOGLE');
+diff --git a/Configure b/Configure
+index 19bed50..e4e4075 100755
+--- Configure
++++ Configure
+@@ -3312,6 +3312,9 @@ EOM
+ 			;;
+ 		next*) osname=next ;;
+ 		nonstop-ux) osname=nonstopux ;;
++		bitrig) osname=bitrig
++			osvers="$3"
++			;;
+ 		openbsd) osname=openbsd
+                 	osvers="$3"
+                 	;;
+BOOGLE
+  }
+  if ( $num < 5.008009 ) {
+  _patch(<<'BITRIGM1');
+diff --git a/Makefile.SH b/Makefile.SH
+index 17298fa..ecaa8ac 100755
+--- Makefile.SH
++++ Makefile.SH
+@@ -77,7 +77,7 @@ true)
+ 	sunos*)
+ 		linklibperl="-lperl"
+ 		;;
+-	netbsd*|freebsd[234]*|openbsd*)
++	netbsd*|freebsd[234]*|openbsd*|bitrig*)
+ 		linklibperl="-L. -lperl"
+ 		;;
+ 	interix*)
+BITRIGM1
+  }
+  else {
+  _patch(<<'BITRIGMX');
+diff --git a/Makefile.SH b/Makefile.SH
+index 17298fa..ecaa8ac 100755
+--- Makefile.SH
++++ Makefile.SH
+@@ -77,7 +77,7 @@ true)
+ 	sunos*)
+ 		linklibperl="-lperl"
+ 		;;
+-	netbsd*|freebsd[234]*|openbsd*|dragonfly*)
++	netbsd*|freebsd[234]*|openbsd*|dragonfly*|bitrig*)
+ 		linklibperl="-L. -lperl"
+ 		;;
+ 	interix*)
+BITRIGMX
+  }
+  if ( $num < 5.008001 ) {
+    # NOOP
+  }
+  elsif ( $num < 5.008007 ) {
+    _patch(<<'BITRIGC3');
+diff --git a/Configure b/Configure
+index 19bed50..e4e4075 100755
+--- Configure	Thu Aug 22 23:20:14 2013
++++ Configure	Thu Aug 22 23:20:35 2013
+@@ -7855,7 +7855,7 @@
+ 	solaris)
+ 		xxx="-R $shrpdir"
+ 		;;
+-	freebsd|netbsd|openbsd)
++	freebsd|netbsd|openbsd|bitrig)
+ 		xxx="-Wl,-R$shrpdir"
+ 		;;
+ 	bsdos|linux|irix*|dec_osf)
+BITRIGC3
+  }
+  elsif ( $num < 5.008009 ) {
+    _patch(<<'BITRIGC2');
+diff --git a/Configure b/Configure
+index 19bed50..e4e4075 100755
+--- Configure	Thu Aug 22 22:56:04 2013
++++ Configure	Thu Aug 22 22:56:25 2013
+@@ -7892,7 +7892,7 @@
+ 	solaris)
+ 		xxx="-R $shrpdir"
+ 		;;
+-	freebsd|netbsd|openbsd|interix)
++	freebsd|netbsd|openbsd|interix|bitrig)
+ 		xxx="-Wl,-R$shrpdir"
+ 		;;
+ 	bsdos|linux|irix*|dec_osf|gnu*)
+BITRIGC2
+  }
+  elsif ( $num < 5.013000 ) {
+    _patch(<<'BITRIGC1');
+diff --git a/Configure b/Configure
+index 19bed50..e4e4075 100755
+--- Configure
++++ Configure
+@@ -8328,7 +8331,7 @@ if "$useshrplib"; then
+ 	solaris)
+ 		xxx="-R $shrpdir"
+ 		;;
+-	freebsd|netbsd|openbsd|interix|dragonfly)
++	freebsd|netbsd|openbsd|interix|dragonfly|bitrig)
+ 		xxx="-Wl,-R$shrpdir"
+ 		;;
+ 	bsdos|linux|irix*|dec_osf|gnu*)
+BITRIGC1
+  }
+  else {
+    _patch(<<'BITRIGCX');
+diff --git a/Configure b/Configure
+index 19bed50..e4e4075 100755
+--- Configure
++++ Configure
+@@ -8328,7 +8331,7 @@ if "$useshrplib"; then
+ 	solaris)
+ 		xxx="-R $shrpdir"
+ 		;;
+-	freebsd|mirbsd|netbsd|openbsd|interix|dragonfly)
++	freebsd|mirbsd|netbsd|openbsd|interix|dragonfly|bitrig)
+ 		xxx="-Wl,-R$shrpdir"
+ 		;;
+ 	bsdos|linux|irix*|dec_osf|gnu*)
+BITRIGCX
+  }
+}
+
+sub _patch_conf_solaris {
+  return unless $^O eq 'solaris';
+  my $perlver = shift;
+  my $num = _norm_ver( $perlver );
+  return unless $num < 5.018000;
+  _patch(<<'BUBBLE');
+diff --git a/Configure b/Configure
+index ff511d3..30ab78a 100755
+--- Configure
++++ Configure
+@@ -8048,7 +8048,20 @@ EOM
+ 			      ;;
+ 			linux|irix*|gnu*)  dflt="-shared $optimize" ;;
+ 			next)  dflt='none' ;;
+-			solaris) dflt='-G' ;;
++			solaris) # See [perl #66604].  On Solaris 11, gcc -m64 on amd64
++				# appears not to understand -G.  gcc versions at
++				# least as old as 3.4.3 support -shared, so just
++				# use that with Solaris 11 and later, but keep
++				# the old behavior for older Solaris versions.
++				case "$gccversion" in
++					'') dflt='-G' ;;
++					*)	case "$osvers" in
++							2.?|2.10) dflt='-G' ;;
++							*) dflt='-shared' ;;
++						esac
++						;;
++				esac
++				;;
+ 			sunos) dflt='-assert nodefinitions' ;;
+ 			svr4*|esix*|nonstopux) dflt="-G $ldflags" ;;
+ 	        *)     dflt='none' ;;
+BUBBLE
+}
+
+#commit 4149c7198d9b78d861df289cce40dd865cab57e7
+sub _patch_regmatch_pointer_5180 {
+  _patch(<<'BOBBLE');
+diff --git a/regexec.c b/regexec.c
+index bc38839..b865b46 100644
+--- regexec.c
++++ regexec.c
+@@ -6662,7 +6662,7 @@ S_regrepeat(pTHX_ regexp *prog, char **startposp, const regnode *p,
+     scan = *startposp;
+     if (max == REG_INFTY)
+ 	max = I32_MAX;
+-    else if (! utf8_target && scan + max < loceol)
++    else if (! utf8_target && loceol - scan > max)
+ 	loceol = scan + max;
+ 
+     /* Here, for the case of a non-UTF-8 target we have adjusted <loceol> down
+@@ -6711,7 +6711,7 @@ S_regrepeat(pTHX_ regexp *prog, char **startposp, const regnode *p,
+ 	    scan = loceol;
+ 	break;
+     case CANY:  /* Move <scan> forward <max> bytes, unless goes off end */
+-        if (utf8_target && scan + max < loceol) {
++        if (utf8_target && loceol - scan > max) {
+ 
+             /* <loceol> hadn't been adjusted in the UTF-8 case */
+             scan +=  max;
+@@ -6730,7 +6730,7 @@ S_regrepeat(pTHX_ regexp *prog, char **startposp, const regnode *p,
+          * can use UTF8_IS_INVARIANT() even if the pattern isn't UTF-8, as it's
+          * true iff it doesn't matter if the argument is in UTF-8 or not */
+         if (UTF8_IS_INVARIANT(c) || (! utf8_target && ! is_utf8_pat)) {
+-            if (utf8_target && scan + max < loceol) {
++            if (utf8_target && loceol - scan > max) {
+                 /* We didn't adjust <loceol> because is UTF-8, but ok to do so,
+                  * since here, to match at all, 1 char == 1 byte */
+                 loceol = scan + max;
+@@ -6910,7 +6910,7 @@ S_regrepeat(pTHX_ regexp *prog, char **startposp, const regnode *p,
+         /* FALLTHROUGH */
+ 
+     case POSIXA:
+-        if (utf8_target && scan + max < loceol) {
++        if (utf8_target && loceol - scan > max) {
+ 
+             /* We didn't adjust <loceol> at the beginning of this routine
+              * because is UTF-8, but it is actually ok to do so, since here, to
+diff --git a/t/re/pat_rt_report.t b/t/re/pat_rt_report.t
+index 2244fdf..9a9b5f5 100644
+--- t/re/pat_rt_report.t
++++ t/re/pat_rt_report.t
+@@ -22,7 +22,7 @@ BEGIN {
+ }
+ 
+ 
+-plan tests => 2530;  # Update this when adding/deleting tests.
++plan tests => 2532;  # Update this when adding/deleting tests.
+ 
+ run_tests() unless caller;
+ 
+@@ -1158,6 +1158,21 @@ EOP
+             '$_ = "abc"; /b/g; $_ = "hello"; print eval q|$\'|,"\n"',
+             "c\n", {}, '$\' first mentioned after match');
+     }
++
++    {
++	# [perl #118175] threaded perl-5.18.0 fails pat_rt_report_thr.t
++	# this tests some related failures
++	#
++	# The tests in the block *only* fail when run on 32-bit systems
++	# with a malloc that allocates above the 2GB line.  On the system
++	# in the report above that only happened in a thread.
++	my $s = "\x{1ff}" . "f" x 32;
++	ok($s =~ /\x{1ff}[[:alpha:]]+/gca, "POSIXA pointer wrap");
++
++	# this one segfaulted under the conditions above
++	# of course, CANY is evil, maybe it should crash
++	ok($s =~ /.\C+/, "CANY pointer wrap");
++    }
+ } # End of sub run_tests
+ 
+ 1;
+BOBBLE
+}
+
+sub _patch_makefile_sh_phony {
+  _patch(<<'END');
+diff --git a/Makefile.SH b/Makefile.SH
+index ac5ade4..8e66603 100755
+--- Makefile.SH
++++ Makefile.SH
+@@ -295,6 +295,30 @@ obj = $(obj1) $(obj2) $(obj3) $(ARCHOBJS)
+ # EMBEDDING is on by default, and MULTIPLICITY doesn't work.
+ #
+
++.PHONY: all compile translators utilities \
++       FORCE \
++       preplibrary \
++       install install-strip install-all install-verbose install-silent \
++       no-install install.perl install.man installman install.html installhtml \
++       check_byacc run_byacc \
++       regen_headers regen_pods regen_all \
++       clean _tidy _mopup _cleaner1 _cleaner2 \
++       realclean _realcleaner clobber _clobber \
++       distclean veryclean _verycleaner \
++       lint \
++       depend \
++       test check test_prep _test_prep \
++       test_tty test-tty _test_tty test_notty test-notty _test_notty \
++       utest ucheck test.utf8 check.utf8 \
++       test.third check.third utest.third ucheck.third test_notty.third \
++       test.deparse test_notty.deparse \
++       minitest \
++       ok okfile oknack okfilenack nok nokfile noknack nokfilenack \
++       clist hlist shlist pllist \
++       distcheck \
++       elc \
++       etags ctags tags
++
+ lintflags = -hbvxac
+
+ .c$(OBJ_EXT):
+END
+}
+
+sub _patch_cow_speed {
+  _patch(<<'COWSAY');
+diff --git a/sv.c b/sv.c
+index 06c0b83..ac1d972 100644
+--- sv.c
++++ sv.c
+@@ -1574,14 +1574,19 @@ Perl_sv_grow(pTHX_ SV *const sv, STRLEN newlen)
+         newlen++;
+ #endif
+ 
++#if defined(PERL_USE_MALLOC_SIZE) && defined(Perl_safesysmalloc_size)
++#define PERL_UNWARANTED_CHUMMINESS_WITH_MALLOC
++#endif
++
+     if (newlen > SvLEN(sv)) {		/* need more room? */
+ 	STRLEN minlen = SvCUR(sv);
+ 	minlen += (minlen >> PERL_STRLEN_EXPAND_SHIFT) + 10;
+ 	if (newlen < minlen)
+ 	    newlen = minlen;
+-#ifndef Perl_safesysmalloc_size
+-        if (SvLEN(sv))
++#ifndef PERL_UNWARANTED_CHUMMINESS_WITH_MALLOC
++        if (SvLEN(sv)) {
+             newlen = PERL_STRLEN_ROUNDUP(newlen);
++        }
+ #endif
+ 	if (SvLEN(sv) && s) {
+ 	    s = (char*)saferealloc(s, newlen);
+@@ -1593,7 +1598,7 @@ Perl_sv_grow(pTHX_ SV *const sv, STRLEN newlen)
+ 	    }
+ 	}
+ 	SvPV_set(sv, s);
+-#ifdef Perl_safesysmalloc_size
++#ifdef PERL_UNWARANTED_CHUMMINESS_WITH_MALLOC
+ 	/* Do this here, do it once, do it right, and then we will never get
+ 	   called back into sv_grow() unless there really is some growing
+ 	   needed.  */
+COWSAY
+}
+
+sub _patch_preprocess_options {
+  my $perl = shift;
+
+  if ($perl =~ /^5\.(?:8|10)\./) {
+    _patch(<<'END');
+diff --git a/perl.c b/perl.c
+index 82e5538..b9e02fe 100644
+--- perl.c
++++ perl.c
+@@ -3758,7 +3758,7 @@ S_open_script(pTHX_ const char *scriptname, bool dosearch, SV *sv,
+ #       ifdef VMS
+             cpp_discard_flag = "";
+ #       else
+-            cpp_discard_flag = "-C";
++            cpp_discard_flag = "-C -ffreestanding";
+ #       endif
+ 
+ #       ifdef OS2
+END
+  } elsif ($perl =~ /^5\.6\./) {
+    _patch(<<'END');
+diff --git a/perl.c b/perl.c
+index 623f9be..014d318 100644
+--- perl.c
++++ perl.c
+@@ -2631,7 +2631,7 @@ sed %s -e \"/^[^#]/b\" \
+  -e '/^#[ 	]*undef[ 	]/b' \
+  -e '/^#[ 	]*endif/b' \
+  -e 's/^[ 	]*#.*//' \
+- %s | %"SVf" -C %"SVf" %s",
++ %s | %"SVf" -C -ffreestanding %"SVf" %s",
+ #  endif
+ #ifdef LOC_SED
+ 	  LOC_SED,
+END
+  }
+}
+
+sub _patch_5183_metajson {
+_patch(<<'DOGSAY');
+diff --git a/META.json b/META.json
+index 64caea7..200e324 100644
+--- META.json
++++ META.json
+@@ -118,7 +118,7 @@
+          "TestInit.pm"
+       ]
+    },
+-   "release_status" : "testing",
++   "release_status" : "stable",
+    "resources" : {
+       "bugtracker" : {
+          "web" : "http://rt.perl.org/perlbug/"
+DOGSAY
+}
+
+sub _norm_ver {
+  my $ver = shift;
+  my @v = split(qr/[._]0*/, $ver);
+  $v[2] ||= 0;
+  return sprintf '%d.%03d%03d', @v;
+}
+
 qq[patchin'];
 
-
-
 __END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Devel::PatchPerl - Patch perl source a la Devel::PPPort's buildperl.pl
 
 =head1 VERSION
 
-version 0.76
+version 1.28
 
 =head1 SYNOPSIS
 
@@ -1848,6 +2662,14 @@ C<perl> version from the specified path.
 If you don't supply the path to unwrapped perl source, it will assume the
 current working directory.
 
+=item C<determine_version>
+
+Takes one optional parameter, the path to unwrapped perl source. It returns the perl version
+of the source code at the given location. It returns undef on error.
+
+If you don't supply the path to unwrapped perl source, it will assume the
+current working directory.
+
 =back
 
 =head1 PLUGIN SYSTEM
@@ -1872,10 +2694,9 @@ Chris Williams <chris@bingosnet.co.uk>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by Chris Williams and Marcus Holland-Moritz.
+This software is copyright (c) 2014 by Chris Williams and Marcus Holland-Moritz.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
